@@ -1,5 +1,6 @@
 import abc
 import datetime
+import hashlib
 import inspect
 import json
 import pickle
@@ -9,6 +10,7 @@ from dataclasses import dataclass, field, fields, is_dataclass
 from decimal import Decimal
 from enum import Enum
 from typing import Any, Generic, Iterable, TypeVar, Union
+from boto3.dynamodb.conditions import Key, Attr
 
 import boto3
 
@@ -122,7 +124,7 @@ class SimulatedTable(TableInterface):
                 item_list.append(item)
         return item_list
 
-    def scan(self):
+    def scan(self, FilterExpression=None):
         scan_response = {
             "Count": len(self._items_as_list()),
             "Items": self._items_as_list(),
@@ -259,6 +261,189 @@ class DynamoDBGlobalParameters:
 
 ###### ---------- Global parameters ---------- ######
 
+###### ---------- Wrapper fo mappable classes ---------- ######
+def _wrap_class(
+
+    cls=None,
+    dynamo_table: str = None,
+    pk: str = None,
+    sk: str = None,
+    logical_table: str = None,
+    unique_id: str = None,
+    pk_name_on_table: str = None,
+    sk_name_on_table: str = None,
+    mapping_schema: dict = None,
+    ignore_attributes: list = None,
+    custom_converter: dict = None,
+):
+    """
+    Adds methods to the class
+    """
+    if hasattr(cls, "dynamo_pk"): # The class has already been decorated
+        return cls
+
+    mapping_schema = mapping_schema if mapping_schema is not None else {}
+    ignore_attributes = ignore_attributes if ignore_attributes is not None else []
+    custom_converter = custom_converter if custom_converter is not None else {}
+    # Generate a logical_table name with low possibility of collision with other classes with the same name
+    dynamo_table = dynamo_table if dynamo_table is not None else cls.__name__
+    logical_table = logical_table if logical_table is not None else cls.__name__[:3]+hashlib.sha1((str(cls.__dict__.keys).encode())).hexdigest()[:15]
+    if unique_id is not None:
+        ignore_attributes.append(str(unique_id))
+    # The methods to be added to the class
+    @classmethod
+    def dynamo_table_name(cls):
+        return dynamo_table
+
+    @classmethod
+    def dynamo_logical_table_name(cls):
+        return logical_table
+
+    @classmethod
+    def dynamo_pk(cls):
+        return pk
+
+    @classmethod
+    def dynamo_sk(cls):
+        return sk
+
+    @classmethod
+    def dynamo_id(cls):
+        return unique_id
+
+    @classmethod
+    def dynamo_pk_name_on_table(cls):
+        return pk_name_on_table
+
+    @classmethod
+    def dynamo_sk_name_on_table(cls):
+        return sk_name_on_table
+
+    @classmethod
+    def dynamo_map(cls):
+        return mapping_schema
+
+    @classmethod
+    def dynamo_ignore(cls):
+        return ignore_attributes
+
+    @classmethod
+    def dynamo_custom_converter(cls):
+        return custom_converter
+
+
+
+    # set the table name
+    cls.dynamo_table_name = dynamo_table_name
+
+    # custom pk and sk
+    if pk is not None:
+        cls.dynamo_pk = dynamo_pk
+        # set the sort key
+        if sk is not None:
+            cls.dynamo_sk = dynamo_sk
+
+    # id and logical table to generate pk and sk
+    else:
+        # set the id
+        cls.dynamo_id = dynamo_id
+        # set the logical table name
+        cls.dynamo_logical_table_name = dynamo_logical_table_name
+        # set the name of the key attributes on the dynamo table
+        cls.dynamo_pk_name_on_table = dynamo_pk_name_on_table
+        cls.dynamo_sk_name_on_table = dynamo_sk_name_on_table
+
+
+    # set the mapping of class attributes to table attributes
+    cls.dynamo_map = dynamo_map
+
+    # set the class attributes that will be ignored (not saved on the database)
+    cls.dynamo_ignore = dynamo_ignore
+
+    if custom_converter is not None:
+        cls.custom_converter = dynamo_custom_converter
+
+    # To add non lambda functions, define the function first them assign it to cls.<function_name>
+
+    return cls
+
+
+def dynamo_entity(
+    cls=None,
+    dynamo_table: str = None,
+    pk: str = None,
+    sk: str = None,
+    logical_table: str = None,
+    unique_id: str = None,
+    pk_name_on_table: str = None,
+    sk_name_on_table: str = None,
+    mapping_schema: dict = None,
+    ignore_attributes: Union[list, tuple] = None,
+    custom_converter: dict = None,
+):
+    """
+    Wraps a class so it is mappable to DynamoDB. Use it as a decorator.
+    The entity class has to allow an empty constructor call.
+
+    DynamoDB uses partition keys (pk) and sort keys (sk) to define a unique data entry,
+    If you are not familiar with DynamoDB, this library can generate this keys if you provide just an id attribute
+    However pay attention to this two rules:
+    - If you provide a pk, the logical table and the id parameters will be ignored.
+    - If you do not provide a pk, it is necessary to provide an id, the pk attribute's name on the database table as
+    pk_name_on_table, and the sk attribute's name on the database table as sk_name_on_table.
+
+    Parameters
+    ----------
+    dynamo_table : str, optional
+        The DynamoDB table name, if no name is provided, the class name will be used. (Default is None)
+    pk : str, optional
+        The name of the attribute that will be used as the partition key on DynamoDB if the pk will be explicit in each
+        object of the decorated class. Using this will ignore the id and logical_table (default is None)
+    sk : Any, optional
+        The name of the attribute that will be used as the sort key if the pk and sk will be explicit in each object of
+        the decorated class. Can be also be None if there is no sort key  (default is None)
+    logical_table: str, optional
+        A custom logical table name to store data on the database, to be used only if the pk parameter is not provided.
+        If no pk and no logical_table parameters are provided, the logical table name will be generated based on the
+        class name, methods and attributes.
+        If you intend to change the decorated class in the future, please provide a logical_table parameter since
+        adding or removing attributes and methods will change the auto-generated logical table name. (default is None)
+    unique_id: str, optional
+        The name of the attribute that will be used as the id, necessary if the pk parameter is not provided.
+        The attribute will be cast to str. (default is None)
+    pk_name_on_table: str, optional
+        The name of the pk attribute on the DynamoDB table, necessary if the pk parameter is not provided.
+        For compatibility with this library, this attribute should have the string (S) type on the database.
+        (default is None)
+    sk_name_on_table: str, optional
+        The name of the sk attribute on the DynamoDB table, necessary if the pk parameter is not provided.
+        For compatibility with this library, this attribute should have the string (S) type on the database.
+        (default is None)
+    mapping_schema: dict, optional
+        A dict mapping the class attributes names to the attributes on DynamoDB.
+        If no mapping is provided for a particular (or all) class attribute, the class attribute name will be used
+        (default is None)
+    ignore_attributes: list[str], optional
+        A list with the name of the class attributes that should not be saved to/loaded from the database.
+        (default is None)
+    custom_converter: dict[str, list[function]]
+        A dict with a custom conversion functions to encode and decode an attribute before saving
+        the keys are the attribute name in the class, the value are lists where list[CrudeRepositoryImp.INSTANCE2ITEM]
+        contains the converter from object to dynamodb item and the list[CrudeRepositoryImp.ITEM2INSTANCE]
+        contains the converter from dynamodb item to object
+    """
+
+    def wrap(cls):
+        return _wrap_class(
+            cls, dynamo_table, pk, sk, logical_table, unique_id, pk_name_on_table, sk_name_on_table, mapping_schema,
+            ignore_attributes, custom_converter
+        )
+
+    if cls is None:
+        return wrap
+
+    return wrap(cls)
+###### ---------- Wrapper fo mappable classes ---------- ######
 
 ###### ---------- Repository Interfaces and Implementation ---------- ######
 
@@ -355,7 +540,32 @@ class DynamoCrudRepository(CrudRepository):
         entity_type: T,
         print_debug: bool = False,
         use_mock_db: bool = False,
+        access_params: dict = None,
     ):
+        """
+        Creates a new DynamoCrudRepository for a specific dynamo_entity class
+
+        Parameters
+        ----------
+        entity_type : Any
+            The class decorated with dynamo_entity that should be mapped to DynamoDB items
+        print_debug : bool, optional
+            To print some debug messages during use
+            (default is False)
+        use_mock_db : bool, optional
+            To use a mock version of DynamoDB instead of trying to access a real DynamoDB (local or AWS hosted)
+            (default is False)
+        access_params: dict, optional
+            A dict with the parameters to access DynamoDB. If not provided, this will be loaded from the aws cli config
+            The dict should have the following format, all keys are mandatory:
+            access_params={
+                'endpoint_url': <string with the endpoint url for the DynamoDB>
+                'region_name': <string with the DynamoDB region name>
+                'aws_access_key_id': <string with the aws access key id to access the DynamoDB>
+                'aws_secret_access_key': <string with the aws secret access key to access the DynamoDB>
+            }
+            (default is None)
+        """
         self.entity_type = entity_type
         self.print_debug = print_debug
 
@@ -371,19 +581,28 @@ class DynamoCrudRepository(CrudRepository):
             self.table_name = self.entity_type.dynamo_table_name()
         if use_mock_db:
             self.ddb = SimulatedDynamoDBResource()
-            if (self.ddb.Table(self.table_name)) is None:
-                self.create_table()  # The table does not exists in the mock dynamodb
+            try:
+                if (self.ddb.Table(self.table_name)) is None:
+                    self.create_table()  # The table does not exists in the mock dynamodb
+            except Exception:  # If self.ddb.Table(table_name) raises an Exception, the table may not exist
+                self.create_table()
         else:
             try:
-                if not global_values.use_aws_cli_credentials:
-                    # Set the db connection
-                    self.ddb = boto3.resource(
-                        "dynamodb",
-                        endpoint_url=global_values.endpoint_url,
-                        region_name=global_values.region_name,  # does not matter locally
-                        aws_access_key_id=global_values.aws_access_key_id,  # does not matter locally
-                        aws_secret_access_key=global_values.aws_secret_access_key,
-                    )
+                if access_params is not None:
+                    # Test if access_params has all the necessary parameters
+                    if {"endpoint_url", "region_name", "aws_access_key_id",
+                        "aws_secret_access_key"} <= set(access_params):
+                        self.ddb = boto3.resource(
+                            "dynamodb",
+                            endpoint_url=access_params["endpoint_url"],
+                            region_name=access_params["region_name"],
+                            aws_access_key_id=access_params["aws_access_key_id"],
+                            aws_secret_access_key=access_params["aws_secret_access_key"],
+                        )
+                    else:
+                        warnings.warn("The provided access_params"  + str(access_params) + "is missing parameters.\n"
+                                      "Trying to use aws cli credentials")
+                        self.ddb = boto3.resource("dynamodb")
                 else:
                     self.ddb = boto3.resource("dynamodb")
             except Exception as ex:
@@ -402,7 +621,6 @@ class DynamoCrudRepository(CrudRepository):
             raise ResourceWarning(warning_str)
 
     def create_table(self):
-        global_values = DynamoDBGlobalParameters.get_instance()
         attribute_definitions, key_schema = self._get_attribute_def_and_key_schema(
             self.entity_type
         )
@@ -412,8 +630,8 @@ class DynamoCrudRepository(CrudRepository):
                 AttributeDefinitions=attribute_definitions,
                 KeySchema=key_schema,
                 ProvisionedThroughput={
-                    "ReadCapacityUnits": global_values.read_capacity_units,
-                    "WriteCapacityUnits": global_values.write_capacity_units,
+                    "ReadCapacityUnits": 5,
+                    "WriteCapacityUnits": 5,
                 },
             )
         except Exception as ex:
@@ -423,16 +641,29 @@ class DynamoCrudRepository(CrudRepository):
 
     def count(self):
         try:
-            scan_response = self.table.scan()
+            if hasattr(self.entity_type, "dynamo_pk"):
+                scan_response = self.table.scan()
+            else:
+                scan_response = self.table.scan(
+                    FilterExpression=Attr(self.entity_type.dynamo_sk_name_on_table()).begins_with(
+                        self.entity_type.dynamo_logical_table_name())
+                )
         except Exception as ex:
             print(ex)
             raise ResourceWarning("Not able to scan table", self.table_name)
         i = scan_response["Count"]
         while "LastEvaluatedKey" in scan_response:
             try:
-                scan_response = self.table.scan(  # TableName='SingleTable',
-                    ExclusiveStartKey=scan_response["LastEvaluatedKey"]
-                )
+                if hasattr(self.entity_type, "dynamo_pk"):
+                    scan_response = self.table.scan(
+                        ExclusiveStartKey=scan_response["LastEvaluatedKey"]
+                    )
+                else:
+                    scan_response = self.table.scan(
+                        FilterExpression=Attr(self.entity_type.dynamo_sk_name_on_table()).begins_with(
+                            self.entity_type.dynamo_logical_table_name()),
+                        ExclusiveStartKey=scan_response["LastEvaluatedKey"]
+                    )
             except Exception as ex:
                 print(ex)
                 warnings.warn("Not able to scan table" + str(self.table_name))
@@ -440,7 +671,37 @@ class DynamoCrudRepository(CrudRepository):
             i += scan_response["Count"]
         return i
 
-    def remove_by_keys(self, keys: dict):
+    def _key_list2key_map(self, keys: Union[dict, list]):
+        if isinstance(keys, list):  # Produce dict from list
+            key_list = keys
+            keys = {
+                self.entity_type.dynamo_pk(): key_list[0],
+                self.entity_type.dynamo_sk(): key_list[1],
+            }
+        return keys
+
+    def _id2key_pair(self, unique_id):
+        sk = self.entity_type.dynamo_logical_table_name()
+        pk = self.entity_type.dynamo_logical_table_name() + "#" + str(unique_id)
+        keys = {
+            self.entity_type.dynamo_pk_name_on_table(): pk,
+            self.entity_type.dynamo_sk_name_on_table(): sk
+        }
+        return keys
+
+    def _pk2id(self, pk):
+        return pk.split('#', 1)[1]
+
+
+    def remove_by_keys(self, keys: Union[dict, list]):
+        """
+        Deleted objects stored in the database using the given keys
+        :param keys: dict, list a set of keys to search for the object.
+         If a list if provided, assumes the pattern [pk, sk]
+        :return: an o object of the mapped class or a list of objects of the mapped class
+        """
+        if isinstance(keys, list):  # Produce dict from list
+            keys = self._key_list2key_map(keys)
         try:
             self.table.delete_item(Key=keys)
         except Exception as ex:
@@ -452,20 +713,32 @@ class DynamoCrudRepository(CrudRepository):
                 + str(self.table_name)
             )
 
+
+    def remove_by_id(self, unique_id):
+        self.remove_by_keys(self._id2key_pair(unique_id))
+
+
+
     def remove(self, to_delete: T):
-        pk_name = self.entity_type.dynamo_pk()
-        pk_value = getattr(to_delete, pk_name)
-        keys = {pk_name: pk_value}
+        if hasattr(self.entity_type,"dynamo_pk"):
+            pk_name = self.entity_type.dynamo_pk_name_on_table()
+            pk_value = getattr(to_delete, pk_name)
+            keys = {pk_name: pk_value}
 
-        if hasattr(self.entity_type, "dynamo_sk"):
-            sk_name = self.entity_type.dynamo_sk()
-            sk_value = getattr(to_delete, sk_name)
-            keys[sk_name] = sk_value
+            if hasattr(self.entity_type, "dynamo_sk"):
+                sk_name = self.entity_type.dynamo_sk_name_on_table()
+                sk_value = getattr(to_delete, sk_name)
+                keys[sk_name] = sk_value
 
-        return self.remove_by_keys(keys)
+            return self.remove_by_keys(keys)
+        else:
+            return self.remove_by_id(getattr(to_delete, to_delete.dynamo_id()))
 
     def item2instance(self, item):
         entity_instance = self.entity_type()
+        if not hasattr(entity_instance, "dynamo_pk"):  # If using id instead of pk and sk
+            pk = item[self.entity_type.dynamo_pk_name_on_table()]
+            setattr(entity_instance, self.entity_type.dynamo_id(), self._pk2id(pk))
         for fl in fields(self.entity_type):
             if fl.name in item:
                 if (
@@ -519,10 +792,7 @@ class DynamoCrudRepository(CrudRepository):
         :return: an o object of the mapped class or a list of objects of the mapped class
         """
         if isinstance(keys, list):  # Produce dict from list
-            key_list = keys
-            keys = {}
-            keys[self.entity_type.dynamo_pk()] = key_list[0]
-            keys[self.entity_type.dynamo_sk()] = key_list[1]
+            keys = self._key_list2key_map(keys)
         try:
             response = self.table.get_item(Key=keys)
 
@@ -581,6 +851,9 @@ class DynamoCrudRepository(CrudRepository):
                 return entity_list if len(entity_list) != 1 else entity_list[0]
             return None
 
+    def find_by_id(self, unique_id):
+        return self.find_by_keys(self._id2key_pair(unique_id))
+
     def keys2KeyConditionExpression(self, keys: dict):
         buffer = ""
         exp_att_val = {}
@@ -596,7 +869,13 @@ class DynamoCrudRepository(CrudRepository):
         entity_list = []
 
         try:
-            scan_response = self.table.scan()
+            if hasattr(self.entity_type, "dynamo_pk"):
+                scan_response = self.table.scan()
+            else:
+                scan_response = self.table.scan(
+                    FilterExpression=Attr(self.entity_type.dynamo_sk_name_on_table()).begins_with(
+                        self.entity_type.dynamo_logical_table_name())
+                )
         except Exception as ex:
             print(ex)
             warnings.warn("Not able to scan table" + str(self.table_name))
@@ -611,9 +890,16 @@ class DynamoCrudRepository(CrudRepository):
 
         while "LastEvaluatedKey" in scan_response:
             try:
-                scan_response = self.table.scan(  # TableName='SingleTable',
-                    ExclusiveStartKey=scan_response["LastEvaluatedKey"]
-                )
+                if hasattr(self.entity_type, "dynamo_pk"):
+                    scan_response = self.table.scan(
+                        ExclusiveStartKey=scan_response["LastEvaluatedKey"]
+                    )
+                else:
+                    scan_response = self.table.scan(
+                        FilterExpression=Attr(self.entity_type.dynamo_sk_name_on_table()).begins_with(
+                            self.entity_type.dynamo_logical_table_name()),
+                        ExclusiveStartKey=scan_response["LastEvaluatedKey"]
+                    )
             except Exception as ex:
                 print(ex)
                 warnings.warn("Not able to scan table" + str(self.table_name))
@@ -627,7 +913,11 @@ class DynamoCrudRepository(CrudRepository):
 
     def instance2item_params(self, obj: T):
         item_params = {}
-
+        if not hasattr(obj, "dynamo_pk"):  # If using id instead of pk and sk
+            sk = obj.dynamo_logical_table_name()
+            pk = obj.dynamo_logical_table_name() + "#" + str(getattr(obj, obj.dynamo_id()))
+            item_params[obj.dynamo_pk_name_on_table()] = pk
+            item_params[obj.dynamo_sk_name_on_table()] = sk
         # Get every attribute of obj, ignoring private members and methods
         for attribute in inspect.getmembers(obj):
             if (
@@ -699,18 +989,28 @@ class DynamoCrudRepository(CrudRepository):
             )
 
     def remove_all(self):
-        pk = self.entity_type.dynamo_pk()
-        sk = self.entity_type.dynamo_sk()
+        if hasattr(self.entity_type, "dynamo_pk"):
+            pk_name = self.entity_type.dynamo_pk()
+            sk_name = self.entity_type.dynamo_sk()
+        else:
+            pk_name = None
+            sk_name = None
         entity_list = self.find_all()
         try:
             with self.table.batch_writer() as batch:
-                for entity in entity_list:
-                    batch.delete_item(
-                        Key={
-                            pk: entity.__getattribute__(pk),
-                            sk: entity.__getattribute__(sk),
-                        }
-                    )
+                if hasattr(self.entity_type, "dynamo_pk"):
+                    for entity in entity_list:
+                        batch.delete_item(
+                            Key={
+                                pk_name: entity.__getattribute__(pk_name),
+                                sk_name: entity.__getattribute__(sk_name),
+                            }
+                        )
+                else:
+                    for entity in entity_list:
+                        batch.delete_item(
+                            Key=self._id2key_pair(str(getattr(entity, entity.dynamo_id())))
+                        )
         except Exception as ex:
             print(ex)
             raise ResourceWarning(
@@ -718,10 +1018,11 @@ class DynamoCrudRepository(CrudRepository):
             )
         return True
 
-    def remove_all_by_keys(self, keys_list: Iterable[dict]):
+    def remove_all_by_keys(self, keys_list: Union[Iterable[dict], Iterable[list]]):
         try:
             with self.table.batch_writer() as batch:
                 for keys in keys_list:
+                    keys = self._key_list2key_map(keys)
                     batch.delete_item(Key=keys)
         except Exception as ex:
             print(ex)
@@ -732,13 +1033,15 @@ class DynamoCrudRepository(CrudRepository):
                 self.table_name,
             )
 
+    def remove_all_by_id(self, unique_id_list: list):
+        keys_list = []
+        for unique_id in unique_id_list:
+            keys_list.append(self._id2key_pair(unique_id))
+        self.remove_all_by_keys(keys_list)
+
     def exists_by_keys(self, keys: Union[dict, list]):
         if isinstance(keys, list):  # Produce dict from list
-            key_list = keys
-            keys = {
-                self.entity_type.dynamo_pk(): key_list[0],
-                self.entity_type.dynamo_sk(): key_list[1],
-            }
+            keys = self._key_list2key_map(keys)
         try:
             response = self.table.get_item(Key=keys)
             if "Item" in response:
@@ -758,6 +1061,9 @@ class DynamoCrudRepository(CrudRepository):
                 return True
         return False
 
+    def exists_by_id(self, unique_id):
+        return self.exists_by_keys(self._id2key_pair(unique_id))
+
     def save_all(self, entities: Iterable[T]):
         try:
             with self.table.batch_writer() as batch:
@@ -776,7 +1082,7 @@ class DynamoCrudRepository(CrudRepository):
             for fl in fls:
                 attrib_type = str
                 if (
-                    fl.name not in self.map_dict
+                    (fl.name not in self.map_dict) and (fl.name not in self.entity_type.dynamo_ignore())
                 ):  # Try to infer the type from the class  attribute type
                     # if using a specific library like numpy or pandas, the user should specify the "N" type himself
                     if issubclass(fl.type, (int, float, Decimal)):
@@ -812,160 +1118,73 @@ class DynamoCrudRepository(CrudRepository):
         attribute_definitions = []
         key_schema = []
         self.map_dict = cls.dynamo_map()
+        if hasattr(cls, "dynamo_pk"):
+            cls_key_attribs = {cls.dynamo_pk(): "HASH"}
+            if hasattr(cls, "dynamo_sk"):
+                cls_key_attribs[cls.dynamo_sk()] = "RANGE"
+            for fl in fields(cls):
+                if fl.name in cls_key_attribs:
+                    # PK and GSI should come before SK and LSI
+                    if cls_key_attribs[fl.name] == "HASH":
+                        attribute_definitions.insert(
+                            0,
+                            {
+                                "AttributeName": "" + fl.name,
+                                "AttributeType": "" + self.map_dict[fl.name],
+                            },
+                        )
+                        key_schema.insert(
+                            0,
+                            {
+                                "AttributeName": "" + fl.name,
+                                "KeyType": cls_key_attribs[fl.name],
+                            },
+                        )
+                    else:
+                        attribute_definitions.append(
+                            {
+                                "AttributeName": "" + fl.name,
+                                "AttributeType": "" + self.map_dict[fl.name],
+                            }
+                        )
+                        key_schema.append(
+                            {
+                                "AttributeName": "" + fl.name,
+                                "KeyType": cls_key_attribs[fl.name],
+                            }
+                        )
+        elif hasattr(cls, "dynamo_pk_name_on_table") and hasattr(cls, "dynamo_sk_name_on_table"):
+            attribute_definitions = [
+                {
+                    "AttributeName": "" + cls.dynamo_pk_name_on_table(),
+                    "AttributeType": "S",
+                },
+                {
+                    "AttributeName": "" + cls.dynamo_sk_name_on_table(),
+                    "AttributeType": "S",
+                }
+            ]
+            key_schema = [
+                {
+                    "AttributeName": "" + cls.dynamo_pk_name_on_table(),
+                    "KeyType": "HASH",
+                },
+                {
+                    "AttributeName": "" + cls.dynamo_sk_name_on_table(),
+                    "KeyType": "RANGE",
+                }
+            ]
+        else:
+            raise RuntimeError("When decorating a class with dynamo_entity, the user needs to pass either a pk "
+                               "parameter or the dynamo_pk_name_on_table, dynamo_sk_name_on_table, and id parameters")
 
-        cls_key_attribs = {cls.dynamo_pk(): "HASH"}
-        if hasattr(cls, "dynamo_sk"):
-            cls_key_attribs[cls.dynamo_sk()] = "RANGE"
 
-        for fl in fields(cls):
-            if fl.name in cls_key_attribs:
-                # PK and GSI should come before SK and LSI
-                if cls_key_attribs[fl.name] == "HASH":
-                    attribute_definitions.insert(
-                        0,
-                        {
-                            "AttributeName": "" + fl.name,
-                            "AttributeType": "" + self.map_dict[fl.name],
-                        },
-                    )
-                    key_schema.insert(
-                        0,
-                        {
-                            "AttributeName": "" + fl.name,
-                            "KeyType": cls_key_attribs[fl.name],
-                        },
-                    )
-                else:
-                    attribute_definitions.append(
-                        {
-                            "AttributeName": "" + fl.name,
-                            "AttributeType": "" + self.map_dict[fl.name],
-                        }
-                    )
-                    key_schema.append(
-                        {
-                            "AttributeName": "" + fl.name,
-                            "KeyType": cls_key_attribs[fl.name],
-                        }
-                    )
         return attribute_definitions, key_schema
 
 
 ###### ---------- Repository Interfaces and Implementation ---------- ######
 
 
-###### ---------- Wrapper fo mappable classes ---------- ######
-def _wrap_class(
-    pk: Any,
-    cls=None,
-    table_name: str = None,
-    sk=None,
-    mapping_schema: dict = None,
-    ignore_attributes: list = None,
-    custom_converter: dict = None,
-):
-    """
-    Adds methods to the class
-    """
-    mapping_schema = mapping_schema if mapping_schema is not None else {}
-    ignore_attributes = ignore_attributes if ignore_attributes is not None else []
-    custom_converter = custom_converter if custom_converter is not None else {}
-    table_name = table_name if table_name is not None else cls.__name__
-
-    # The methods to be added to the class
-    @classmethod
-    def dynamo_table_name(cls):
-        return table_name
-
-    @classmethod
-    def dynamo_pk(cls):
-        return pk
-
-    @classmethod
-    def dynamo_sk(cls):
-        return sk
-
-    @classmethod
-    def dynamo_map(cls):
-        return mapping_schema
-
-    @classmethod
-    def dynamo_ignore(cls):
-        return ignore_attributes
-
-    @classmethod
-    def dynamo_custom_converter(cls):
-        return custom_converter
-
-    # set the table name
-    cls.dynamo_table_name = dynamo_table_name
-
-    # set the partition key
-    cls.dynamo_pk = dynamo_pk
-
-    # set the sort key
-    if sk is not None:
-        cls.dynamo_sk = dynamo_sk
-
-    # set the mapping of class attributes to table attributes
-    cls.dynamo_map = dynamo_map
-
-    # set the class attributes that will be ignored (not saved on the database)
-    if ignore_attributes is not None:
-        cls.dynamo_ignore = dynamo_ignore
-
-    if custom_converter is not None:
-        cls.custom_converter = dynamo_custom_converter
-
-    # To add non lambda functions, define the function first them assign it to cls.<function_name>
-
-    return cls
-
-
-def dynamo_entity(
-    pk: Any,
-    cls=None,
-    table_name: str = None,
-    sk=None,
-    mapping_schema: dict = None,
-    ignore_attributes: list = None,
-    custom_converter: dict = None,
-):
-    """
-    Wraps a class so it is mappable to DynamoDB. Use it as a decorator.
-    The entity class has to allow an empty constructor call.
-
-    Parameters
-    ----------
-    pk : Any
-        The name of the attribute that will be used as the partition key on DynamoDB
-    table_name : str, optional
-        The DynamoDB table name, if no name is provided, the class name will be used
-    sk : Any, optional
-        The sort key used with the partition key. Can be None if there is no sort key (default is None)
-    mapping_schema: dict, optional
-        A dict mapping the class attributes names to the attributes on DynamoDB.
-        If no mapping is provided for a particular (or all) class attribute, the class attribute name will be used
-        (default is None)
-    ignore_attributes: list[str], optional
-        A list with the name of the class attributes that should not be saved to/loaded from the database.
-        (default is None)
-    custom_converter: dict[str, list[function]]
-        A dict with a custom conversion functions to encode and decode an attribute before saving
-        the keys are the attribute name in the class, the value are lists where list[CrudeRepositoryImp.INSTANCE2ITEM]
-        contains the converter from object to dynamodb item and the list[CrudeRepositoryImp.ITEM2INSTANCE]
-        contains the converter from dynamodb item to object
-    """
-
-    def wrap(cls):
-        return _wrap_class(
-            pk, cls, table_name, sk, mapping_schema, ignore_attributes, custom_converter
-        )
-
-    if cls is None:
-        return wrap
-
-    return wrap(cls)
 
 
 ###### ---------- Example ---------- ######
@@ -983,10 +1202,12 @@ class Counter:
 
 
 @dataclass
-@dynamo_entity(table_name="dTableName", pk="pk", sk="sk")
+@dynamo_entity(dynamo_table="dTableName", unique_id="int_id", pk_name_on_table="pk", sk_name_on_table="sk")
 class SampleEntityClass:
-    pk: str = None
-    sk: int = None
+    # pk: str = None
+    # sk: int = None
+    # str_id: int = None
+    int_id: int = None
     attrib1: str = "This is the attribute 1"
     attrib2: int = 2
     field1: float = field(default=0.0)
@@ -1007,45 +1228,40 @@ if __name__ == "__main__":
 
     aMap = SampleEntityClass.dynamo_map()
 
-    aObj = SampleEntityClass(pk="aPartitionKey", sk=counter.get_next())
+    aObj = SampleEntityClass(int_id=counter.get_next())
 
     print("Saving object:", aObj, "\n")
     sample_crud.save(aObj)
 
-    anotherObj = sample_crud.find_by_keys(
-        {"pk": "aPartitionKey", "sk": counter.get_current()}
-    )
+    anotherObj = sample_crud.find_by_id(counter.get_current())
     print("Loaded object:", anotherObj, "\n")
 
     print("Updated attribute")
     aObj.attrib2 = 20
     sample_crud.save(aObj)
-    anotherObj = sample_crud.find_by_keys({"pk": "aPartitionKey"})
+    anotherObj = sample_crud.find_by_id(counter.get_current())
     print("Loaded object:", anotherObj, "\n")
 
     sample_crud.remove(aObj)
     print("Object deleted")
 
-    anotherObj = sample_crud.find_by_keys({"pk": "aPartitionKey"})
+    anotherObj = sample_crud.find_by_id(counter.get_current())
     print("Loaded object:", anotherObj, "\n")
 
     n_items = sample_crud.count()
     aList = []
     for _ in range(100):
-        aList.append(SampleEntityClass(pk="aPartitionKey", sk=counter.get_next()))
+        aList.append(SampleEntityClass(int_id=counter.get_next()))
     sample_crud.save_all(aList)
     n_items = sample_crud.count()
     bList = sample_crud.find_all()
-    aTruth = sample_crud.exists_by_keys(
-        {"pk": "aPartitionKey", "sk": counter.get_current()}
-    )
-    sample_crud.remove_all_by_keys(
-        [{"pk": "aPartitionKey", "sk": counter.get_current()}]
-    )
+    aTruth = sample_crud.exists_by_id(counter.get_current())
+    sample_crud.remove_all_by_id([counter.get_current()])
     aLie = sample_crud.exists_by_keys({"pk": "aPartitionKey", "sk": counter.get_next()})
     sample_crud.remove_all()
     print(sample_crud.count())
 
+
     # Na classe de teste tipos variados do python , tipos como decimal, float, int, string, bool, map, datetime(string),
     # enum,
     # Implementar as classes de exemplo
